#========================================================================================
# §package      DE_EPAGES::Ebay::API::Cartridge
# §base         DE_EPAGES::Design::API::DesignInstaller
# §state        public
#----------------------------------------------------------------------------------------
# §description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Ebay::API::Cartridge;
use base DE_EPAGES::ShopMailType::API::ShopMailTypeInstaller;

use strict;
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Core::API::File qw ( GetFileContent SetAccessRights );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Config qw( GetSQLiteDBFile );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle RunOnStore );
use DE_EPAGES::Database::API::DBInterface qw ( ProcessSqlGo );
use DE_EPAGES::Database::API::OrderFile qw ( GetSqlFileOrder );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw ( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

#========================================================================================
# §function     new
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Installer = DE_EPAGES::Ebay::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# §description  Creates a new installer object of DE_EPAGES::Ebay.
#----------------------------------------------------------------------------------------
# §input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# §return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Ebay',
        'Version'            => '7.69.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# §function     install
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# §description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;

    ImportObjects( $self->xmlDirectory .'/EbayEnvironment.xml' );

    # import regions
    ImportObjects( $self->xmlDirectory . '/EbayRegions.xml');
    # import call types
    ImportObjects( $self->xmlDirectory . '/EbayCallTypes.xml');
    # import ebay system templates
    ImportObjects( $self->xmlDirectory . '/DefaultSystemTemplate.xml');
    # import ebay mail types
    ImportObjects( $self->xmlDirectory . '/MailTypes.xml');

    my $EbayCategoriesDB = GetSQLiteDBFile('EbayCategories') // EBAY_CATEGORIES_DB_FILENAME;
    $self->installEbayCategories(EBAY_CATEGORIES_DB_NAME, $EbayCategoriesDB);
    $self->finishInstall;
    return;
}

sub installEbayCategories {
    my $self = shift;
    my ($DBName, $DBFilename, $ForceUpdate) = @_;

    RunOnStore(
        Store => $DBName,
        Sub => sub  {
            my $dbi = GetCurrentDBHandle();

            if (not $dbi->existsTable('ebaysite') or $ForceUpdate) {
                my $Dir = join '/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory , 'Database', $dbi->driverName;
                my $aFiles = GetSqlFileOrder( $Dir );
                foreach my $File (@$aFiles) {
                    next if $File =~ /\/delete_/;
                    my $rContent = GetFileContent( $File );
                    ProcessSqlGo( $$rContent );
                }
                SetAccessRights( $DBFilename );
            }
        }
    );
}

#========================================================================================
# §function     uninstall
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# §description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;

    # EbayCallTypes.xml and EbayRegions.xml are cleaned up
    # on uninstall by deleting the respective database tables
    $self->call( sub { DeleteObjects( $self->xmlDirectory . '/MailTypes.xml') } );
    $self->call( sub { DeleteObjects( $self->xmlDirectory .'/DefaultSystemTemplate.xml' ) } );
    $self->call( sub { DeleteObjects( $self->xmlDirectory .'/EbayEnvironment.xml' ) } );

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# §function     encryptData
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->encryptData;
#----------------------------------------------------------------------------------------
# §description  Encrpyts data of this cartridge.
#========================================================================================
sub encryptData {
    my $self = shift;

    $self->encryptDataByClass('EbayShopEnvironment');

    GetLog()->info('encrypting ebay attributes of System class');
    my $System = LoadRootObject();

    my @EncrpytedAttributes = qw (
        EbayCertId EbayProviderPassword EbayProviderToken EbayRuName EbaySessionID
        EbaySandBoxCertId EbaySandBoxProviderPassword EbaySandBoxProviderToken EbaySandBoxRuName
    );

    $System->set( $System->get(\@EncrpytedAttributes) );

}

#========================================================================================
# §function     patchDBStructure_6_16_0
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_16_0;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.16.0' to version '6.16.1'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.16.1' | string
#========================================================================================
sub patchDBStructure_6_16_0 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebaytransaction.sql
        unless ($dbi->existsTableColumn('ebaytransaction', 'subproductauctionid')) {
            $self->logInfo('add column ebaytransaction.subproductauctionid');
            $dbi->addTableColumnAfterColumn('ebaytransaction', 'feedbacktext', 'subproductauctionid integer null');
        }
        $self->logInfo('create index ebaytransaction.i_ebaytrans_subproductauction');
        $dbi->createIndex('ebaytransaction', 'i_ebaytrans_subproductauction', undef, ['subproductauctionid'])
            unless ($dbi->existsIndex('ebaytransaction','i_ebaytrans_subproductauction'));

        if (not $dbi->existsForeignKey('fk_ebaytransaction_subproductauctionid')) {
            $self->logInfo('add foreign key ebaytransaction.fk_ebaytransaction_subproductauctionid');
            $dbi->addForeignKey(
                'ebaytransaction',
                'fk_ebaytransaction_subproductauctionid',
                [
                    'subproductauctionid',
                ],
                'ebayauction',
                [
                    'auctionid',
                ]
            );
        }

        #changes for file /Database/mysql/Tables/ebayauction.sql
        unless ($dbi->existsTableColumn('ebayauction', 'parentauctionid')) {
            $self->logInfo('add column ebayauction.parentauctionid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'usertemplateonly', 'parentauctionid integer null');
        }
        $self->logInfo('create index ebayauction.i_auction_parentauctionid');
        $dbi->createIndex('ebayauction', 'i_auction_parentauctionid', undef, ['parentauctionid'])
            unless ($dbi->existsIndex('ebayauction','i_auction_parentauctionid'));

        if (not $dbi->existsForeignKey('fk_ebayauction_parentauctionid')) {
            $self->logInfo('add foreign key ebayauction.fk_ebayauction_parentauctionid');
            $dbi->addForeignKey(
                'ebayauction',
                'fk_ebayauction_parentauctionid',
                [
                    'parentauctionid',
                ],
                'ebayauction',
                [
                    'auctionid',
                ]
            );
        }

    }

    return '6.16.1';
}

#========================================================================================
# §function     patchDBStructure_6_17_6
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_17_6;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.6' to version '6.17.7'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.7' | string
#========================================================================================
sub patchDBStructure_6_17_6 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebayauction.sql
        unless ($dbi->existsTableColumn('ebayauction', 'isreviseinventory')) {
            $self->logInfo('add column ebayauction.isreviseinventory after column ebayauction.parentauctionid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'parentauctionid', 'isreviseinventory bool not null default 0');
            $dbi->modifyTableColumnDropDefault('ebayauction','isreviseinventory');
        }

    }

    return '6.17.7';
}

#========================================================================================
# §function     patchDBStructure_6_17_7
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_6_17_7;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.7' to version '6.17.8'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '6.17.8' | string
#========================================================================================
sub patchDBStructure_6_17_7 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebayauction.sql
        unless ($dbi->existsTableColumn('ebayauction', 'isbestofferenabled')) {
            $self->logInfo('add column ebayauction.isbestofferenabled after column ebayauction.isreviseinventory');
            $dbi->addTableColumnAfterColumn('ebayauction', 'isreviseinventory', 'isbestofferenabled bool not null default 0');
            $dbi->modifyTableColumnDropDefault('ebayauction','isbestofferenabled');
        }
        unless ($dbi->existsTableColumn('ebayauction', 'bestofferautoacceptprice')) {
            $self->logInfo('add column ebayauction.bestofferautoacceptprice');
            $dbi->addTableColumnAfterColumn('ebayauction', 'isbestofferenabled', 'bestofferautoacceptprice decimal(19,4) null');
        }
        unless ($dbi->existsTableColumn('ebayauction', 'minimumbestofferprice')) {
            $self->logInfo('add column ebayauction.minimumbestofferprice');
            $dbi->addTableColumnAfterColumn('ebayauction', 'bestofferautoacceptprice', 'minimumbestofferprice decimal(19,4) null');
        }
    }

    return '6.17.8';
}

#========================================================================================
# §function     patchDBStructure_7_2_0
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_7_2_0;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '7.2.0' to version '7.3.0'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '7.3.0' | string
#========================================================================================
sub patchDBStructure_7_2_0 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content


    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebayauction.sql
        unless ($dbi->existsTableColumn('ebayauction', 'returnswithin')) {
            $self->logInfo('add column ebayauction.returnswithin');
            $dbi->addTableColumnAfterColumn('ebayauction', 'minimumbestofferprice', 'returnswithin varchar(255) null');
        }

    }

    return '7.3.0';
}

#========================================================================================
# §function     patchDBStructure_7_9_0
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Installer->patchDBStructure_7_9_0;
#----------------------------------------------------------------------------------------
# §description  Patches cartridge db structure in active store (database connection)
#               from version '7.9.0' to version '7.10.0'.
#----------------------------------------------------------------------------------------
# §return       $Version | version of cartridge after patch function, '7.10.0' | string
#========================================================================================
sub patchDBStructure_7_9_0 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content
    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebayauction.sql
        unless ($dbi->existsTableColumn('ebayauction', 'businesspolicypaymentid')) {
            $self->logInfo('add column ebayauction.businesspolicypaymentid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'returnswithin', 'businesspolicypaymentid varchar(30) null');
        }
        unless ($dbi->existsTableColumn('ebayauction', 'businesspolicyshippingid')) {
            $self->logInfo('add column ebayauction.businesspolicyshippingid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'businesspolicypaymentid', 'businesspolicyshippingid varchar(30) null');
        }
        unless ($dbi->existsTableColumn('ebayauction', 'businesspolicyreturnpolicyid')) {
            $self->logInfo('add column ebayauction.businesspolicyreturnpolicyid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'businesspolicyshippingid', 'businesspolicyreturnpolicyid varchar(30) null');
        }

    }

    return '7.10.0';
}

1;
