#========================================================================================
# package      DE_EPAGES::Presentation::API::Object::TrayBuilder
# state        public
#----------------------------------------------------------------------------------------
# description  Builder for new object tray introduced in 6.17.0
#========================================================================================
package DE_EPAGES::Presentation::API::Object::TrayBuilder;

use strict;

use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestArray);
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Builder = DE_EPAGES::Presentation::API::Object::TrayBuilder->new($hOptions);
#----------------------------------------------------------------------------------------
# example
#           $Builder = DE_EPAGES::Presentation::API::Object::TrayBuilder->new( {
#                    LanguageID         => $LanguageID,
#                    TranslationPackage => 'DE_EPAGES::Presentation'
#                   }
#           );
#----------------------------------------------------------------------------------------
# description  Constructor of TrayBuilder object.
#----------------------------------------------------------------------------------------
# input        $hOptions |
#               <ul>
#                   <li> LanguageID | required for translation column names | integer
#                   <li> TranslationPackage | required for translation column names | string
#               </ul>| hash.ref.
#----------------------------------------------------------------------------------------
# return       $Builder | builder object | DE_EPAGES::Presentation::API::Object::TrayBuilder
#========================================================================================
sub new {
    my $class = shift;
    my ($hParam) = @_;

    TestHash(
        'ConstructorArguments',
        $hParam, {
            LanguageID         => 'Integer;NotNull',
            TranslationPackage => 'String;NotNull'
        }
    );

    my $hArgs = {};
    $hArgs->{_Header}             = [];
    $hArgs->{_LanguageID}         = $hParam->{LanguageID};
    $hArgs->{_TranslationPackage} = $hParam->{TranslationPackage};

    return bless $hArgs, $class;
}

#========================================================================================
# function     addColumnHeader
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Builder->addColumnHeader($Id, $aFlags, $IsSortable, $TranslationKey)
#----------------------------------------------------------------------------------------
# description  Add a new coulmn header to list.
#----------------------------------------------------------------------------------------
# input        $Id | unique name to identify this column | string
# input        $aFlags |
#                  <ul>
#                   <li>$IsSelectable - column will be decorated with checkbox in gui - boolean</li>
#                   <li>$IsSortable - column will be decorated with sort icon in gui - boolean</li>
#                   <li>$IsLinkable - show column as text - boolean</li>
#                   <li>$IsRightAligned - set alignment to right - boolean</li>
#                  </ul>   | array.ref.
## input       $TranslationKey | optional - key used localization will result in {foo},
#                                 if dictionary don't know 'foo'. set undef to omit headline text | string
# input        $TranslationPackage | optional - overwrite default translation package for this column | string
# return       $self | self reference | DE_EPAGES::Presentation::API::Object::TrayBuilder
#========================================================================================
sub addColumnHeader {
    my $self = shift;
    my ( $Id, $aFlags, $TranslationKey, $TranslationPackage ) = @_;
    TestArray('aFlags', $aFlags);
    my ($IsSelectable, $IsSortable, $IsLinkable, $IsRightAligned) = @$aFlags;

    my $aHeader = $self->_getHeader();
    my $TransPkg = $TranslationPackage ? $TranslationPackage : $self->_getTranslationPackage();
    my $Translation = $TranslationKey ? Translate( $TranslationKey, $self->_getLanguageID(), $TransPkg ) : '';

    push @$aHeader, {
        'id'             => $Id,
        'name'           => $Translation,
        'isSelectable'   => $IsSelectable,
        'isSortable'     => $IsSortable,
        'isLinkable'     => $IsLinkable,
        'isRightAligned' => $IsRightAligned,
    };

    return $self;
}

#========================================================================================
# function     getHeaderList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aHeader = $Builder->getHeaderList()
#----------------------------------------------------------------------------------------
# description  Returns list of column header. See <function addColumnHeader>
#----------------------------------------------------------------------------------------
# return       $aHeader | list of hashes:
#                <ul>
#                    <li>id - unique id - string
#                    <li>name - localized name - string
#                    <li>isSelectable - can column be used to select items - boolean
#                    <li>isSortable - can column be used to sort items - boolean
#                </ul>  | array.ref
#========================================================================================
sub getHeaderList {
    my $self = shift;
    return $self->_getHeader();
}

sub _getHeader {
    return shift->{_Header};
}

sub _getLanguageID {
    return shift->{_LanguageID};
}

sub _getTranslationPackage {
    return shift->{_TranslationPackage};
}

1;
