#========================================================================================
# package      DE_EPAGES::ProductPortal::t::PortalSitesByCountry
#----------------------------------------------------------------------------------------
# description  Test TLEs processing related to ProductPortals
#========================================================================================
package DE_EPAGES::ProductPortal::t::PortalSitesByCountry;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More;

use DE_EPAGES::ProductPortal::API::TLE::GetPortalSitesByCountry;
use DE_EPAGES::TLE::API::BaseProcessor;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

sub test {
    my $self = shift;

    $self->testGetPortalSitesByCountry();
    $self->testGetPortalSitesByCountryIsOnlyAmazon();
    $self->testGetPortalSitesByCountryIsOnlyGoogle();
    $self->testGetPortalSitesByCountryIsGoogleSmartShoppingNotInList();

    done_testing();
}


#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------

sub testGetPortalSitesByCountry {
    my $self = shift;

    note 'Test GET_PORTAL_SITES_BY_COUNTRY';
    #Set languageID for localized attribute NameOrAlias
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $self->shop->get('LanguageID');

    my $Processor = $self->_setUp();

    my $fPortalSitesByCountry = sub {
        my ($ShopID) = @_;
        $Processor->callBlock('GET_PORTAL_SITES_BY_COUNTRY', sub {[ $ShopID ]});
    };


    my $PortalSitesByCountry = $fPortalSitesByCountry->($self->shop->id);

    ok(ref($PortalSitesByCountry) eq "ARRAY", 'Expecting to return PortalSitesByCountry ref array for the ShopID');

    cmp_ok(scalar @{$PortalSitesByCountry}, '>=',1, 'Expecting Number of PortalSites to be greater than zero');

    my $PortalSite = shift (@{$PortalSitesByCountry});
    ok(defined( $PortalSite),'Expecting PortalSite to be defined');
    ok(exists( $PortalSite->{'ProductPortalName'}),'Expecting ProductPortalName key to exist');
    ok(exists( $PortalSite->{'PortalSiteIsMultiLanguage'}),'Expecting PortalSiteIsMultiLanguage key to exist');
}

sub testGetPortalSitesByCountryIsOnlyAmazon {
    my $self = shift;
    #Set languageID for localized attribute NameOrAlias
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $self->shop->get('LanguageID');
    my $Processor = $self->_setUp();
    $self->shop->set({'IsUnityRegistered' => 1});

    my $fPortalSitesByCountry = sub {
        my ($ShopID) = @_;
        my $isOnlyAmazon = 1;
        $Processor->callBlock('GET_PORTAL_SITES_BY_COUNTRY', sub {[ $ShopID, $isOnlyAmazon ]});
    };

    my $PortalSitesByCountry = $fPortalSitesByCountry->($self->shop->id);

    $self->shop->set({'IsUnityRegistered' => 0});

    is(scalar @$PortalSitesByCountry, 1, 'Expecting only one entry to be returned');
    is($PortalSitesByCountry->[0]->{'FeatureAlias'}, 'Amazon', 'Expecting only Amazon to be returned');
}

sub testGetPortalSitesByCountryIsOnlyGoogle {
    my $self = shift;
    #Set languageID for localized attribute NameOrAlias
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $self->shop->get('LanguageID');
    my $Processor = $self->_setUp();
    $self->shop->set({'IsUnityRegistered' => 1});

    my $fPortalSitesByCountry = sub {
        my ($ShopID) = @_;
        my $isOnlyGoogle = 1;

        $Processor->callBlock('GET_PORTAL_SITES_BY_COUNTRY', sub {[ $ShopID, undef, $isOnlyGoogle ]});
    };

    my $PortalSitesByCountry = $fPortalSitesByCountry->($self->shop->id);

    $self->shop->set({'IsUnityRegistered' => 0});

    is(scalar @$PortalSitesByCountry, 1, 'Expecting only one entry to be returned');
    is($PortalSitesByCountry->[0]->{'FeatureAlias'}, 'GoogleBase', 'Expecting only Google to be returned');
}

sub testGetPortalSitesByCountryIsGoogleSmartShoppingNotInList {
    my $self = shift;
    #Set languageID for localized attribute NameOrAlias
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $self->shop->get('LanguageID');
    my $Processor = $self->_setUp();
    $self->shop->set({'IsUnityRegistered' => 1});

    my $fPortalSitesByCountry = sub {
        my ($ShopID) = @_;

        $Processor->callBlock('GET_PORTAL_SITES_BY_COUNTRY', sub {[ $ShopID, undef, undef ]});
    };

    my $PortalSitesByCountry = $fPortalSitesByCountry->($self->shop->id);

    my $AliasGoogleSmartShopping = 'GoogleSmartShopping';
    my $GoogleSmartShopping;
    foreach my $PortalSite (@$PortalSitesByCountry) {
        if ($PortalSite->{'FeatureAlias'} eq $AliasGoogleSmartShopping) {
            $GoogleSmartShopping = $PortalSite->{'FeatureAlias'};
        }
    }
    $self->shop->set({'IsUnityRegistered' => 0});

    is($GoogleSmartShopping, undef, 'Expecting GoogleSmartShopping not to be in list');
}

#--------------------------------------------------------------------------------------------

sub _setUp {
    my $self = shift;
    # Build a wrapper function to use the TLE function "GET_PORTAL_SITES_BY_COUNTRY"
    my $Processor = DE_EPAGES::TLE::API::BaseProcessor->new();
    my $PortalSites = DE_EPAGES::ProductPortal::API::TLE::GetPortalSitesByCountry->new();
    $PortalSites->register($Processor);

    return $Processor
}

__PACKAGE__->runScript;
