#========================================================================================
# package      DE_EPAGES::ShopMigration::API::ShopCommonData
# state        public
#----------------------------------------------------------------------------------------
# description common (shop) functions used by multiple exporters/importers
#========================================================================================
package DE_EPAGES::ShopMigration::API::ShopCommonData;
use base qw(Exporter);

use strict;
use feature 'switch';
our @EXPORT_OK = qw(
    GetBasicShopData
    SetBasicShopData
    GetActiveShopLanguages
    SetShopData
    GetShopData
    AddDefaultWidgetsUnity
);

use DE_EPAGES::Core::API::Log qw(LogDebug);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Object::API::Factory qw(LoadObject Transaction LoadRootObject ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode GetCodeByLanguageID);
use DE_EPAGES::Shop::API::Table::ShopCountry qw(GetInfoShopCountriesByShopID);
use DE_EPAGES::ShopMigration::API::Constants qw(
    BASIC_SHOP_DATA_ATTRIBUTES_GET
    BASIC_SHOP_DATA_ATTRIBUTES_SET
    BASIC_SHOP_DATA_LOCALIZEDATTRIBUTES
    SHOP_DATA_ATTRIBUTES_FOR_BASE_EXPORT
    SHOP_DATA_LOCALIZEDATTRIBUTES_FOR_BASE_EXPORT
    SHOP_DATA_SCRIBEATTRIBUTES_FOR_BASE_EXPORT
    SHOP_DATA_OBJECTATTRIBUTES_FOR_BASE_EXPORT
    SHOP_DATA_ATTRIBUTES_FOR_PREP_EXPORT
    SHOP_DATA_LOCALIZEDATTRIBUTES_FOR_PREP_EXPORT
    SHOP_DATA_OBJECTATTRIBUTES_FOR_PREP_EXPORT
);
use DE_EPAGES::ShopMigration::API::HTMLSanitizer qw(SanitizeHTMLForScribe);
use DE_EPAGES::ShopMigration::API::ShortURL qw(InsertRedirect GetAllUILocalesForShop);
use DE_EPAGES::Tax::API::Table::TaxRate qw(ExistsTaxRate InsertTaxRate UpdateTaxRate GetInfoTaxRatesByTaxAreaID); ##no critic (RequireCorrectImports)
use DE_EPAGES::Object::API::Table::Locale qw(GetLanguageByLocaleID);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'REFERENCED_OBJECT_NOT_FOUND' => 'Cannot assign value to attribute #Name as #Path was not found below root or shop',
    };
}
#ERRORDEFINITION

#=========================================================================================#=============================
# function     GetBasicShopData
# state        public
#-----------------------------------------------------------------------------------------------------------------------
# syntax       $hShopData = GetBasicShopData($Shop);
#-----------------------------------------------------------------------------------------------------------------------
# description  gets all data which is needed to create the prepshop for a migration shop
#-----------------------------------------------------------------------------------------------------------------------
# input        $Shop | migration shop object | object
# return       $hShopData | shop data. used keys are:
#               <ul>
#                 <li>Attributes - hash ref with following keys</li>
#                   <ul>
#                     <li>Alias                   - string</li>
#                     <li>WebServerScriptNamePart - string</li>
#                     <li>CurrencyID              - string</li>
#                     <li>LocaleID                - string</li>
#                     <li>TaxModel                - boolean</li>
#                     <li>InputTaxModel           - boolean</li>
#                     <li>VATreleased             - boolean</li>
#                   </ul>
#                 </li>
#                 <li>ActiveLanguages - return of DE_EPAGES::ShopMigration::API::ShopCommonData::GetActiveShopLanguages</li>
#                 <li>LocalizedAttributes - hash ref with keys language
#                   <ul>
#                     <li>LanguageCode2             - en|de|..
#                       <ul>
#                         <li>Name                  - string</li>
#                       </ul>
#                     </li>
#                   </ul>
#                 </li>
#                 <li>ShopCountriesData - return of DE_EPAGES::ShopMigration::API::ShopCommonData::GetShopCountriesData</li>
#                 <li>RegionsData - return of DE_EPAGES::ShopMigration::API::ShopCommonData::GetShopRegionsData</li>
#                 <li>AddressData - return of DE_EPAGES::ShopMigration::API::ShopCommonData::GetShopAddressData</li>
#                 <li>TaxMatrixData - return of DE_EPAGES::ShopMigration::API::ShopCommonData::GetShopTaxMatrixData</li>
#               </ul> | ref.hash
#=======================================================================================================================
sub GetBasicShopData {
    my ($Shop) = @_;

    my $hShopData;
    #baseattributes
    $hShopData->{'Attributes'} = $Shop->get(BASIC_SHOP_DATA_ATTRIBUTES_GET);

    $hShopData->{'ActiveLanguages'} = GetActiveShopLanguages($Shop);
    #localized attributes
    foreach my $hLanguage (@{$hShopData->{'ActiveLanguages'}}) {
        my $hLocalInfo = $Shop->get(BASIC_SHOP_DATA_LOCALIZEDATTRIBUTES, $hLanguage->{'LanguageID'});
        $hShopData->{'LocalizedAttributes'}{$hLanguage->{'Code2'}} = $hLocalInfo;
    }

    $hShopData->{'ShopCountriesData'} = GetShopCountriesData($Shop);
    $hShopData->{'RegionsData'} = GetShopRegionsData($Shop);
    $hShopData->{'AddressData'} = GetShopAddressData($Shop);
    $hShopData->{'TaxMatrixData'} = GetShopTaxMatrixData($Shop);

    LogDebug('GetBasicShopData hShopData of ' . $Shop->pathString, $hShopData);
    return $hShopData;
}

#========================================================================================
# function     SetBasicShopData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetBasicShopData($PrepShop, $hShopData);
#----------------------------------------------------------------------------------------
# description  set initial data to prepshop (with data of GetBasicShopData)
#----------------------------------------------------------------------------------------
# input        $PrepShop     | shop object | object
# input        $hShopData    | hash ref | hash see GetBasicShopData
#========================================================================================
sub SetBasicShopData {
    my ($PrepShop, $hShopData) = @_;
    LogDebug('SetBasicShopData hShopData of ' . $PrepShop->pathString, $hShopData);

    #baseattributes
    #copy hash
    my %Info = %{$hShopData->{'Attributes'}};
    foreach my $key (keys %Info) {
        delete $Info{$key} unless ElementOf($key, @{BASIC_SHOP_DATA_ATTRIBUTES_SET()});
    }
    $PrepShop->set(\%Info);

    #correct default locales/languages/currencies
    _correctLanguages($PrepShop);
    _correctLocales($PrepShop);
    _correctCurrencies($PrepShop);

    SetActiveShopLanguages($PrepShop, $hShopData->{'ActiveLanguages'});
    #localized attributes
    foreach my $LanguageCode2 (keys %{$hShopData->{LocalizedAttributes}}) {
        #copy hash
        my %LangInfo = %{$hShopData->{LocalizedAttributes}{$LanguageCode2}};
        foreach my $key (keys %LangInfo) {
            delete $LangInfo{$key} unless ElementOf($key, @{BASIC_SHOP_DATA_ATTRIBUTES_SET()});
        }
        my $LanguageID = GetPKeyLanguageByCode($LanguageCode2);
        # Silently ignore fields for non-existing languages
        if ($PrepShop->existsLanguage($LanguageID)) {
            $PrepShop->set(\%LangInfo, $LanguageID);
        }
    }

    SetShopCountriesData($PrepShop, $hShopData->{'ShopCountriesData'});
    SetShopRegionsData($PrepShop, $hShopData->{'Regions'});
    SetShopAddressData($PrepShop, $hShopData->{'AddressData'});
    SetShopTaxMatrixData($PrepShop, $hShopData->{'TaxMatrixData'});
    return;
}

#========================================================================================
# function     GetShopData
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetShopData($Shop);
#----------------------------------------------------------------------------------------
# description  creates a hash with relevant fields to be set later with SetShopData
#               ( Attributes and LocalizedAttributes )
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# return       $hShopData    | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li> SHOP_DATA_ATTRIBUTES_FOR_BASE_EXPORT (if Shop is BaseShop)
#                           <li> SHOP_DATA_ATTRIBUTES_FOR_PREP_EXPORT (if Shop is PrepShop)
#                           <li> SHOP_DATA_OBJECTATTRIBUTES_FOR_BASE_EXPORT  (if Shop is BaseShop)
#                           <li> SHOP_DATA_OBJECTATTRIBUTES_FOR_PREP_EXPORT  (if Shop is PrepShop)
#                       </ul>
#                   <li>LocalizedAttributes
#                       <li>LanguageCode2  (en|de|..)
#                           <ul>
#                               <li> see SHOP_DATA_LOCALIZEDATTRIBUTES_FOR_BASE_EXPORT (if Shop is BaseShop)
#                               <li> see SHOP_DATA_LOCALIZEDATTRIBUTES_FOR_PREP_EXPORT (if Shop is PrepShop)
#                           </ul>
#                       </ul>
#               </ul>| reference to hash
#========================================================================================
sub GetShopData {
    my ($Shop) = @_;

    my $UsePrepAttributes = $Shop->get('IsMigrationPrepShop');
    my ($Attributes, $ObjectAttributes, $LocalizedAttributes, $ScribeAttributes) =
        _selectAttributesArrayRefsForShopData($UsePrepAttributes);
    # We map 'MinOrderAmounts' to 'MinOrderAmount'
    $Attributes = [ grep {$_ ne 'MinOrderAmount'} @$Attributes ];
    my $hValues = { Attributes => $Shop->get($Attributes), LocalizedAttributes => {}, };
    $hValues->{Attributes}{MinOrderAmount} = _getDefaultMinOrderAmount($Shop);
    foreach my $AttributeName (@$ObjectAttributes) {
        my $Value = $hValues->{Attributes}{$AttributeName};
        next unless defined $Value;
        if ($Shop->class->attribute($AttributeName)->getSite() == $Shop) {
            $hValues->{Attributes}{$AttributeName} = $Value->relativePath($Shop);
        } else {
            $hValues->{Attributes}{$AttributeName} = $Value->pathString();
        }
    }
    my $ActiveLanguages = GetActiveShopLanguages($Shop);
    foreach my $hLanguage (@$ActiveLanguages) {
        my $AttrInfo = $Shop->get($LocalizedAttributes, $hLanguage->{'LanguageID'});
        foreach my $FieldName (@$ScribeAttributes) {
            $AttrInfo->{$FieldName} = SanitizeHTMLForScribe($AttrInfo->{$FieldName});
        }
        $hValues->{LocalizedAttributes}{$hLanguage->{'Code2'}} = $AttrInfo;
    }
    return $hValues;
}

sub _getDefaultMinOrderAmount {
    my ($Shop) = @_;

    my $MinOrderAmounts = $Shop->get('MinOrderAmounts');
    my $CurrencyID = $Shop->get('CurrencyID');
    my $MinOrderAmount;
    foreach my $Entry (@$MinOrderAmounts) {
        if ($Entry->{'CurrencyID'} eq $CurrencyID) {
            $MinOrderAmount = $Entry->{'Value'};
            last;
        }
    }
    return $MinOrderAmount;
}

#========================================================================================
# function     SetShopData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopData($Shop, $hShopData);
#----------------------------------------------------------------------------------------
# description  sets attributes on a shop as specified in the hash $hShopData
#               ( Attributes and LocalizedAttributes )
#               use GetShopData() to create a proper hash
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hShopData    | hash ref | hash see GetShopData
#========================================================================================
sub SetShopData {
    my ($Shop, $hShopData) = @_;

    my $UsePrepAttributes = $Shop->get('IsMigrationPrepShop') ? 0 : 1;
    my ($Attributes, $ObjectAttributes, $LocalizedAttributes) = _selectAttributesArrayRefsForShopData($UsePrepAttributes);
    foreach my $Type (keys %$hShopData) {
        given ($Type) {
            when ('Attributes') {
                foreach my $AttributeName (keys %{$hShopData->{Attributes}}) {
                    die "Invalid Element for Attributes: $AttributeName" unless ElementOf($AttributeName, @$Attributes);
                    # Reverse the mapping from 'MinOrderAmounts' to 'MinOrderAmount'
                    if ($AttributeName eq 'MinOrderAmount') {
                        $hShopData->{Attributes}{MinOrderAmounts} =
                            [ { CurrencyID => $Shop->get('CurrencyID'), Value => $hShopData->{Attributes}{MinOrderAmount} } ];
                        delete $hShopData->{Attributes}{MinOrderAmount};
                    }
                }
                foreach my $AttributeName (@$ObjectAttributes) {
                    my $PathString = $hShopData->{Attributes}{$AttributeName};
                    next unless defined $PathString;
                    if (ExistsObjectByPath($PathString)) {
                        $hShopData->{Attributes}{$AttributeName} = LoadObjectByPath($PathString);
                    } elsif (ExistsObjectByPath($PathString, $Shop->id)) {
                        $hShopData->{Attributes}{$AttributeName} = $Shop->path($PathString);
                    } else {
                        Error('REFERENCED_OBJECT_NOT_FOUND', { Name => $AttributeName, Path => $PathString });
                    }
                    if ($Shop->class->attribute($AttributeName)->getSite() == $Shop) {
                        $hShopData->{Attributes}{$AttributeName} = $Shop->path($PathString);
                    } elsif (ExistsObjectByPath($PathString)) {
                        $hShopData->{Attributes}{$AttributeName} = LoadObjectByPath($PathString);
                    }
                }
                $Shop->set($hShopData->{Attributes});
            }
            when ('LocalizedAttributes') {
                foreach my $LanguageCode2 (keys %{$hShopData->{LocalizedAttributes}}) {
                    foreach my $Key (keys %{$hShopData->{LocalizedAttributes}{$LanguageCode2}}) {
                        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$LocalizedAttributes);
                    }
                    my $LanguageID = GetPKeyLanguageByCode($LanguageCode2);
                    # Silently ignore fields for non-existing languages
                    if ($Shop->existsLanguage($LanguageID)) {
                        $Shop->set($hShopData->{LocalizedAttributes}{$LanguageCode2}, $LanguageID);
                    }
                }
            } default {
                die "Invalid Type: $Type";
            }
        }
    }

    # add redirect entries for each locale (de_DE..) to epages-ui
    my $IsBaseShop = $Shop->get('IsMigrationPrepShop') ? 0 : 1;
    if ($IsBaseShop) {
        my $aLocales = GetAllUILocalesForShop($Shop);
        foreach my $LocaleID (@$aLocales) {
            my $LanguageID = GetLanguageByLocaleID($LocaleID);
            my $FromSlug = $LocaleID;
            if (defined $FromSlug) {
                my $ToSlug = '/' . GetCodeByLanguageID($LanguageID);
                InsertRedirect($Shop, $FromSlug, $ToSlug);
            }
        }
    }
    return;
}

#========================================================================================
# function     GetActiveShopLanguages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLanguages = GetActiveShopLanguages($Shop);
#----------------------------------------------------------------------------------------
# description  gets all activated languages for a shop
#----------------------------------------------------------------------------------------
# input        %$Shop | shop object | object
# return       $ahActiveLanguages | array ref of active language hashes with following keys
#               <ul>
#                 <li>Code2      - string</li>
#                 <li>LanguageID - integer</li>
#               </ul> | ref.array.hash
#========================================================================================
sub GetActiveShopLanguages {
    my ($Shop) = @_;

    my $aSiteLocales = $Shop->get('SiteLocales');
    my @UsedLanguageIDs;
    my @ActiveLanguages;
    foreach my $SiteLocale (sort {$a->get('Language')->{'Code2'} cmp $b->get('Language')->{'Code2'}} @$aSiteLocales) {
        my $LanguageID = $SiteLocale->get('Language')->{'LanguageID'};
        next if $LanguageID ~~ @UsedLanguageIDs;
        my $Code2 = $SiteLocale->get('Language')->{'Code2'};
        push @ActiveLanguages, {
            'Code2'      => $Code2,
            'LanguageID' => $LanguageID
        };
        push @UsedLanguageIDs, $LanguageID;
    }
    return \@ActiveLanguages;
}

#========================================================================================
# function     SetActiveShopLanguages
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetActiveShopLanguages($PrepShop, $ahActiveLanguages);
#----------------------------------------------------------------------------------------
# description  activates languages for a shop
#----------------------------------------------------------------------------------------
# input        $PrepShop     | shop object | object
# input        $ahActiveLanguages  | see GetActiveShopLanguages | ref.array.hash
#========================================================================================
sub SetActiveShopLanguages {
    my ($PrepShop, $ahActiveLanguages) = @_;

    #add additional languages/locales
    foreach my $hLanguage (@$ahActiveLanguages) {
        my $LanguageID = GetPKeyLanguageByCode($hLanguage->{'Code2'});
        unless ($PrepShop->existsLanguage($LanguageID)) {
            my $NewLanguage;
            Transaction(sub {
                $NewLanguage = $PrepShop->addLanguage($LanguageID);
            });
            my $LocaleIDForNewShop = $PrepShop->localeForLanguageByShopLocaleIDAndLanguage($NewLanguage);
            # now add the matching locale to the shop
            unless ($PrepShop->existsLocale($LocaleIDForNewShop)) {
                $PrepShop->addLocale($LocaleIDForNewShop);
            }
        }
    }
    return;
}

#========================================================================================
# function     GetShopCountriesData
# state        public
# syntax       $ahCountries = GetShopCountriesData($Shop);
#----------------------------------------------------------------------------------------
# description  returns all countries of the shop with position and isdefault
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#----------------------------------------------------------------------------------------
# return       $ahCountries | array ref of all shop countries with following keys
#               <ul>
#                 <li>CountryID - integer</li>
#                 <li>Position  - integer</li>
#                 <li>IsDefault - boolean</li>
#               </ul> | ref.array.hash
#========================================================================================
sub GetShopCountriesData {
    my ($Shop) = @_;

    my $ahShopCountries = GetInfoShopCountriesByShopID($Shop->id);
    return [ map {
        { 'CountryID'   => $_->{'CountryID'},
            'Position'  => $_->{'Position'},
            'IsDefault' => $_->{'IsDefault'},
        }
    } sort {$a->{'Position'} <=> $b->{'Position'}} @$ahShopCountries ];
}

#========================================================================================
# function     SetShopCountriesData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopCountriesData($PrepShop, $ahCountries);
#----------------------------------------------------------------------------------------
# description  adds/removes countries for a shop
#----------------------------------------------------------------------------------------
# input        $PrepShop     | shop object | object
# input        $ahCountries  | see GetShopCountriesData | ref.array.hash
#========================================================================================
sub SetShopCountriesData {
    my ($PrepShop, $ahCountries) = @_;

    #delete existing countries
    foreach my $hCountry (@{$PrepShop->get('Countries')}) {
        $PrepShop->removeCountry($hCountry->{'CountryID'});
    }
    #add all countries of hash
    foreach my $Country (@{$ahCountries}) {
        $PrepShop->addCountry($Country->{CountryID}, $Country->{Position}, $Country->{IsDefault}) unless $PrepShop->existsCountry($Country->{CountryID});
    }
    return;
}

#========================================================================================
# function     GetShopRegionsData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahRegions = GetShopRegionsData($Shop)
#----------------------------------------------------------------------------------------
# description  get shop regions with their countries (if there are any)
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#----------------------------------------------------------------------------------------
# return       $ahRegions | array ref of all shop regions with following keys
#               <ul>
#                 <li>Alias      - integer</li>
#                 <li>Countries  - ref.array of CountryIDs </li>
#               </ul> | ref.array.hash
#========================================================================================
sub GetShopRegionsData {
    my ($Shop) = @_;

    my $aRegions = $Shop->get('Regions');
    my @Regions = map {
        {
            'Alias'     => $_->alias,
            'Countries' => _getRegionCountries($_),
        }
    } @$aRegions;
    return \@Regions;
}

#========================================================================================
# function     SetShopRegionsData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopRegionsData($PrepShop, $ahRegions);
#----------------------------------------------------------------------------------------
# description  add all regions, region countries
#----------------------------------------------------------------------------------------
# input        $PrepShop     | shop object | object
# input        $ahRegions    | see GetShopRegionsData | ref.array.hash
#========================================================================================
sub SetShopRegionsData {
    my ($PrepShop, $ahRegions) = @_;

    my $RegionsFolder = $PrepShop->folder('Regions');
    #delete existing regions
    foreach my $Region (@{$RegionsFolder->children()}) {
        $Region->delete;
    }
    #add all regions and region countries
    foreach my $hRegion (@{$ahRegions}) {
        my $Alias = $hRegion->{'Alias'};
        my $Region = $RegionsFolder->existsChild($Alias) ? $RegionsFolder->child($Alias) : $PrepShop->addRegion($Alias);
        $Region->saveCountries($hRegion->{'Countries'});
    }
    return;
}

#========================================================================================
# function     GetShopAddressData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hShopAddressData = GetShopAddressData($Shop)
#----------------------------------------------------------------------------------------
# description  gets shop address data
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#----------------------------------------------------------------------------------------
# return       $hShopAddressData | hash ref of address with following keys
#               <ul>
#                 <li>Alias              - string</li>
#                 <li>City               - string</li>
#                 <li>CommercialRegister - string</li>
#                 <li>Company            - string</li>
#                 <li>CountryID          - integer</li>
#                 <li>EMail              - string</li>
#                 <li>Fax                - string</li>
#                 <li>FirstName          - string</li>
#                 <li>LastName           - string</li>
#                 <li>Phone              - string</li>
#                 <li>State              - string</li>
#                 <li>Street2            - string</li>
#                 <li>VATID              - string</li>
#                 <li>Zipcode            - string</li>
#               </ul> | ref.hash
#========================================================================================
sub GetShopAddressData {
    my ($Shop) = @_;

    my @ShopAddressAttributes = qw(Alias Company FirstName LastName EMail Phone Fax CountryID Street Street2 Zipcode City State VATID CommercialRegister);
    my $Address = $Shop->get('Address');
    return $Address->get(\@ShopAddressAttributes);
}

#========================================================================================
# function     SetShopAddressData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopAddressData($PrepShop, $hShopDataAddress);
#----------------------------------------------------------------------------------------
# description  set shop address data to prepshop
#----------------------------------------------------------------------------------------
# input        $PrepShop         | shop object | object
# input        $hShopAddressData | hash see GetShopAddressData | ref.hash
#========================================================================================
sub SetShopAddressData {
    my ($PrepShop, $hShopDataAddress) = @_;
    my $Address = $PrepShop->get('Address');
    $Address->set($hShopDataAddress);
}

#=========================================================================================#=============================
# function     GetShopTaxMatrixData
# state        public
#-----------------------------------------------------------------------------------------------------------------------
# syntax       $hShopTaxMatrixData = GetShopTaxMatrixData($Shop);
#-----------------------------------------------------------------------------------------------------------------------
# description  gets all data which is needed to set tax settings
#-----------------------------------------------------------------------------------------------------------------------
# input        $Shop | migration shop object | object
# return       $hShopTaxMatrixData | shop taxmatrix data, used keys are:
#               <ul>
#                 <li>TaxMatrix - string (alias of taxmatrix)</li>
#                 <li>TaxMatrixIsCustomized - boolean</li>
#                 <li>ShopTaxMatrix
#                       - return of DE_EPAGES::ShopMigration::API::ShopCommonData::GetCustomizedTaxMatrixData only
#                         if it is a customized shop taxmatrix
#                 </li>
#               </ul> | ref.hash
#=======================================================================================================================
sub GetShopTaxMatrixData {
    my ($Shop) = @_;

    my $hInfo;
    my $TaxMatrix = $Shop->get('TaxMatrix');
    $hInfo->{'TaxMatrix'} = $TaxMatrix->get('RelativePath');

    if ($TaxMatrix->parent->id != $Shop->id) {
        #system taxmatrix is used
        $hInfo->{'TaxMatrixIsCustomized'} = 0;
    } else {
        #shoptaxmatrix is used
        $hInfo->{'TaxMatrixIsCustomized'} = 1;
        $hInfo->{'ShopTaxMatrix'} = GetCustomizedTaxMatrixData($Shop);
    }
    return $hInfo;
}

#========================================================================================
# function     GetCustomizedTaxMatrixData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hCustomizedShopTaxMatrixData = GetCustomizedTaxMatrixData($Shop);
#----------------------------------------------------------------------------------------
# description  gets all relevant data of a customized shop taxmatrix
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $hCustomizedShopTaxMatrixData | taxmatrix attributes, used keys are:
#               <ul>
#                 <li>Alias - string (alias of taxmatrix)</li>
#                 <li>DefaultTaxClass - string     - alias of DefaultTaxClass</li>
#                 <li>DefaultTaxArea  - string     - alias of DefaultTaxArea</li>
#                 <li>FallbackTaxArea - string     - alias of FallbackTaxArea</li>
#                 <li>TaxClasses      - ref.array  - aliases of tax classes</li>
#                 <li>TaxAreas        - ref.array  - aliases of tax areas</li>
#                 <li>TaxRates        - ref.array.hash - array of tax rates with keys:
#                   <ul>
#                     <li>TaxArea     - string - alias of taxarea
#                     <li>TaxClass    - string - alias of taxclass
#                     <li>TaxRate     - float
#                   </ul>
#                 </li>
#               </ul> | ref.hash
#========================================================================================
sub GetCustomizedTaxMatrixData {
    my ($Shop) = @_;

    my $TaxMatrix = $Shop->get('TaxMatrix');
    my %Info;
    $Info{'Alias'} = $TaxMatrix->alias;
    my @TaxMatrixAttributes = qw(DefaultTaxClass DefaultTaxArea FallbackTaxArea);
    my $hTaxMatrixInfo = $TaxMatrix->get(\@TaxMatrixAttributes);
    foreach my $Attribute (@TaxMatrixAttributes) {
        $Info{$Attribute} = $hTaxMatrixInfo->{$Attribute}->alias;
    }

    my $aTaxClasses = $TaxMatrix->get('TaxClasses');
    $Info{'TaxClasses'} = [ map {$_->alias} @$aTaxClasses ];

    my $aTaxAreas = $TaxMatrix->get('TaxAreas');
    $Info{'TaxAreas'} = [ map {$_->alias} @$aTaxAreas ];

    my @Items;
    foreach my $TaxArea (@$aTaxAreas) {
        foreach my $hTaxRate (@{GetInfoTaxRatesByTaxAreaID($TaxArea->id)}) {
            push @Items, {
                'TaxArea'  => $TaxArea->alias,
                'TaxClass' => LoadObject($hTaxRate->{'TaxClassID'})->alias,
                'TaxRate'  => $hTaxRate->{'Rate'},
            };
        }
    }
    $Info{'TaxRates'} = \@Items;
    return \%Info;
}

#========================================================================================
# function     SetShopTaxMatrixData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopTaxMatrixData($Shop, $hShopTaxMatrixData);
#----------------------------------------------------------------------------------------
# description  sets the shop taxmatrix,
#               calls SetCustomizedTaxMatrixData if taxmatrix is customized
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $hShopTaxMatrixData | hash see GetShopTaxMatrixData | ref.hash
#========================================================================================
sub SetShopTaxMatrixData {
    my ($PrepShop, $hShopTaxMatrixData) = @_;

    if ($hShopTaxMatrixData->{'TaxMatrixIsCustomized'}) {
        SetCustomizedTaxMatrixData($PrepShop, $hShopTaxMatrixData->{'ShopTaxMatrix'});
    } else {
        my $TaxMatrix = LoadRootObject()->child($hShopTaxMatrixData->{'TaxMatrix'});
        $PrepShop->set({ 'TaxMatrix' => $TaxMatrix });
    }
}

#=====================================================================================================
# function     SetCustomizedTaxMatrixData
# state        public
#-----------------------------------------------------------------------------------------------------
# syntax       SetCustomizedTaxMatrixData($Shop, $hCustomizedShopTaxMatrixData);
#-----------------------------------------------------------------------------------------------------
# description  creates a customized shop taxmatrix if it does not exist
#-----------------------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $hCustomizedShopTaxMatrixData | hash see GetCustomizedTaxMatrixData | ref.hash.string
#=====================================================================================================
sub SetCustomizedTaxMatrixData {
    my ($Shop, $hShopTaxMatrix) = @_;

    my $ShopTaxMatrix;
    unless ($Shop->existsChild($hShopTaxMatrix->{'Alias'})) {
        $ShopTaxMatrix = $Shop->insertChild('TaxMatrix', {
            'Alias' => $hShopTaxMatrix->{'Alias'},
        });
        foreach my $Alias (@{$hShopTaxMatrix->{'TaxClasses'}}) {
            $ShopTaxMatrix->insertChild('TaxClass', {
                'Alias' => $Alias,
            });
        }
        foreach my $Alias (@{$hShopTaxMatrix->{'TaxAreas'}}) {
            $ShopTaxMatrix->insertChild('TaxArea', {
                'Alias' => $Alias,
            });
        }
    } else {
        $ShopTaxMatrix = $Shop->child($hShopTaxMatrix->{'Alias'});
    }
    # set defaults
    $ShopTaxMatrix->set({ 'DefaultTaxClass' => $ShopTaxMatrix->child($hShopTaxMatrix->{'DefaultTaxClass'}) }) if defined $hShopTaxMatrix->{'DefaultTaxClass'};
    $ShopTaxMatrix->set({ 'DefaultTaxArea' => $ShopTaxMatrix->child($hShopTaxMatrix->{'DefaultTaxArea'}) }) if defined $hShopTaxMatrix->{'DefaultTaxArea'};
    $ShopTaxMatrix->set({ 'FallbackTaxArea' => $ShopTaxMatrix->child($hShopTaxMatrix->{'FallbackTaxArea'}) }) if defined $hShopTaxMatrix->{'FallbackTaxArea'};

    foreach my $hTaxRate (@{$hShopTaxMatrix->{'TaxRates'}}) {
        my $TaxAreaID = $ShopTaxMatrix->child($hTaxRate->{'TaxArea'})->id;
        my $TaxClassID = $ShopTaxMatrix->child($hTaxRate->{'TaxClass'})->id;
        if (ExistsTaxRate($TaxAreaID, $TaxClassID)) {
            UpdateTaxRate({ 'TaxAreaID' => $TaxAreaID, 'TaxClassID' => $TaxClassID, 'Rate' => $hTaxRate->{'TaxRate'} });
        } else {
            InsertTaxRate({ 'TaxAreaID' => $TaxAreaID, 'TaxClassID' => $TaxClassID, 'Rate' => $hTaxRate->{'TaxRate'} });
        }
    }
    $Shop->set({ 'TaxMatrix' => $ShopTaxMatrix });
    return;
}

# Helper function that copies the proper (constant) ArraysRefs depending to use on Get/Set
# 1 -> use Constants named *PREP* (PrepExport and BaseImport)
# 0 -> use Constants named *BASE* (BaseExport and PrepImport)
sub _selectAttributesArrayRefsForShopData {
    my ($UsePrepAttributes) = @_;

    my ($Attributes, $ObjectAttributes, $LocalizedAttributes, $ScribeAttributes);
    if ($UsePrepAttributes) {
        $Attributes = [ @{SHOP_DATA_ATTRIBUTES_FOR_PREP_EXPORT()} ];
        $ObjectAttributes = [ @{SHOP_DATA_OBJECTATTRIBUTES_FOR_PREP_EXPORT()} ];
        $LocalizedAttributes = [ @{SHOP_DATA_LOCALIZEDATTRIBUTES_FOR_PREP_EXPORT()} ];
        $ScribeAttributes = [];
    } else {
        $Attributes = [ @{SHOP_DATA_ATTRIBUTES_FOR_BASE_EXPORT()} ];
        $ObjectAttributes = [ @{SHOP_DATA_OBJECTATTRIBUTES_FOR_BASE_EXPORT()} ];
        $LocalizedAttributes = [ @{SHOP_DATA_LOCALIZEDATTRIBUTES_FOR_BASE_EXPORT()} ];
        $ScribeAttributes = [ @{SHOP_DATA_SCRIBEATTRIBUTES_FOR_BASE_EXPORT()} ];
    }
    return ($Attributes, $ObjectAttributes, $LocalizedAttributes, $ScribeAttributes);
}

#gets all countries of a region
sub _getRegionCountries {
    my ($Region) = @_;
    return [ map {$_->{'CountryID'}} @{$Region->get('Countries')} ]
}

#removes all languages except the default one
sub _correctLanguages {
    my ($PrepShop) = @_;

    #decrease current counter of languages (to add a language also with maxvalue=1)
    $PrepShop->featureIncrementValue('Languages', -1);

    my $CurrentLanguageID = $PrepShop->get('LanguageID');
    $PrepShop->addLanguage($CurrentLanguageID) unless ($PrepShop->existsLanguage($CurrentLanguageID));
    foreach my $ShopLanguageID (@{$PrepShop->languages}) {
        if ($CurrentLanguageID != $ShopLanguageID) {
            $PrepShop->removeLanguage($ShopLanguageID);
        }
    }

    #increase current counter of languages by 1
    $PrepShop->featureIncrementValue('Languages');
    return;
}

#removes all locales except the default one
sub _correctLocales {
    my ($PrepShop) = @_;

    my $CurrentLocaleID = $PrepShop->get('LocaleID');
    unless ($PrepShop->existsLocale($CurrentLocaleID)) {
        $PrepShop->addLocale($CurrentLocaleID);
    }
    foreach my $ShopLocale (@{$PrepShop->get('SiteLocales')}) {
        if ($CurrentLocaleID ne $ShopLocale->alias) {
            $PrepShop->removeLocale($ShopLocale->alias);
        }
    }
    return;
}

#removes all currencies except the default one
sub _correctCurrencies {
    my ($PrepShop) = @_;

    my $CurrentCurrencyID = $PrepShop->get('CurrencyID');
    unless ($PrepShop->existsCurrency($CurrentCurrencyID)) {
        $PrepShop->addCurrency($CurrentCurrencyID);
        $PrepShop->activateCurrency($CurrentCurrencyID);
    }
    foreach my $ShopCurrencyID (@{$PrepShop->currencies}) {
        if ($CurrentCurrencyID ne $ShopCurrencyID) {
            $PrepShop->removeCurrency($ShopCurrencyID);
        }
    }
    return;
}

#========================================================================================
# function     AddDefaultWidgetsUnity
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddDefaultWidgetsUnity($Shop);
#----------------------------------------------------------------------------------------
# description  adds the default widget for Unity
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub AddDefaultWidgetsUnity {
    my ($Shop) = @_;

    # same as in BasicUnity.xml
    my $aDefaultWidgetTypes = [ {
        Name     => 'SalesVolumeUnity',
        Position => 50,
    } ];

    my $aMerchants = $Shop->child('Groups')->child('Merchants')->directMembers;
    my $WidgetTypesFolder = LoadRootObject()->child('WidgetTypes');

    foreach my $User (@$aMerchants) {
        my $WidgetFolder = $User->folder('MBOWidgets');
        foreach my $Widget (@$aDefaultWidgetTypes) {
            my $Alias = $Widget->{Name};
            my $Position = $Widget->{Position};
            if (!$WidgetTypesFolder->existsChild($Alias)) {
                LogDebug("Widget $Alias does not exists.", 1);
                next;
            }
            my $WidgetType = $WidgetTypesFolder->child($Alias);
            my $Feature = $WidgetType->get('Feature');
            if (!defined $Feature || $Shop->featureMaxValue($Feature)) {
                $WidgetFolder->insertChild('Widget', {
                    'WidgetType' => $WidgetType,
                    'Alias'      => $WidgetType->alias,
                    'Position'   => $Position
                }) unless $WidgetFolder->existsChild($WidgetType->alias);
            }
        }
    }
}

1;
